#!/usr/bin/perl
#
# Script to create parameter files from I1.scm

use strict;

my $paramfl = "I1.scm";

mkdir("epsilon",0777);
mkdir("extended",0777);
mkdir("interval",0777);
mkdir("kahan",0777);
mkdir("nothing",0777);
mkdir("offset",0777);
mkdir("offsetwgt0",0777);
mkdir("string",0777);
mkdir("warn",0777);

# Set up parameter files for contrasting G = 0.4 with G = 0.5
# value for subdir key => [Class, [class parameters], initial wealth]
my %dirsettingspt5
  = ("epsilon" => ["DoubleEpsilon", ['~0.0', '~4.49', '~4.5'], '0.0'],
     "extended" => ["DoubleLong", [], '0.0'],
     "interval" => ["DoubleIntervalWarn", [], '0.0'],
     "kahan" => ["DoubleKahanSum", [], '0.0'],
     "nothing" => ["DoubleSimple", [], '0.0'],
     "offset" => ["DoubleOffset", ['~0.0', '~3.6028797018963968e+16',
				   '~7.2057594037927936e+16'], '0.0'],
     "offsetwgt0" => ["DoubleOffset", ['~0.0', '~3.6028797018963968e+16',
				       '~7.2057594037927936e+16'], '1.0'],
     "string" => ["DoubleString", ['s1', 's2', 's25', 's50'], '0.0'],
     "warn" => ["DoubleWarn", ['!ALL'], '0.0']);

my %dirsettingspt4
  = ("epsilon" => ["DoubleEpsilon",
		   ['~0.0', '~8.8817841970012523233890533447265625e-16',
		    '~3.59', '~3.6'], '0.0'],
     "extended" => ["DoubleLong", [], '0.0'],
     "interval" => ["DoubleIntervalWarn", [], '0.0'],
     "kahan" => ["DoubleKahanSum", [], '0.0'],
     "nothing" => ["DoubleSimple", [], '0.0'],
     "offset" => ["DoubleOffset", ['~8.0', '~16.0', '~1.8014398509481984e+16',
				   '~3.6028797018963968e+16'], '0.0'],
     "offsetwgt0" => ["DoubleOffset",
		      ['~0.0', '~5.5511151231257827021181583404541015625e-17',
		       '~1.8014398509481984e+16', '~3.6028797018963968e+16'],
		      '1.0'],
     "string" => ["DoubleString", ['s1', 's2', 's15', 's16'], '0.0'],
     "warn" => ["DoubleWarn", ['!ALL'], '0.0']);

foreach my $Gsettings ([\%dirsettingspt4, "0.4", "3.6"],
		       [\%dirsettingspt5, "0.5", "4.5"]) {
  my $settings = $$Gsettings[0];
  my $G = $$Gsettings[1];
  my $J = $$Gsettings[2];

  print "Creating parameter files for G = $G\n";
  while(my ($subdir, $sdsettings) = each(%$settings)) {
    my $class = $$sdsettings[0];
    my $classparams = $$sdsettings[1];
    my $initwealth = $$sdsettings[2];
    
    print "Creating parameter files in subdir $subdir\n";
    if($#$classparams == -1) {
      &create_param_fl($paramfl, $subdir, $class, "0", $G, $J, $initwealth);
      next;
    }
    elsif($#$classparams == 0) {
      &create_param_fl($paramfl, $subdir, $class, $$classparams[0], $G, $J,
		       $initwealth);
      next;
    }
    foreach my $classparam (@$classparams) {
      &create_param_fl($paramfl, $subdir, $class, $classparam, $G, $J,
		       $initwealth);
    }
  }
}

# Create parameter files for G = 0.01i, i = 0, 99. This is to confirm the same
# behaviour in the model as is observed using the program I1 in the same
# directory as this script.

print "Creating parameter files for G = 0.01i, i = 0-99\n";
for(my $i = 0; $i <= 99; $i++) {
  my $G = sprintf("0.%02d", $i);
  &create_param_fl($paramfl, "nothing", "DoubleSimple", "0", $G, "0.0");
}

print "Done\n";
exit 0;

sub create_param_fl {
  my ($orig, $subdir, $class, $cparam, $G, $J, $w0) = @_;

  open(ORIG, "<$orig") or die "Error reading base parameter file $orig: $!\n";

  my $newfl = "G=$G";
  if($cparam ne "0") {
    my $i = 1;

    while(-e "$subdir/$newfl-$i.scm") {
      $i++;
    }
    $newfl .= "-$i";
  }
  $newfl .= ".scm";

  if(-e "$subdir/$newfl") {
    warn "New parameter file $subdir/$newfl unexpectedly exists\n";
  }

  open(NEW, ">$subdir/$newfl")
    or die "Error creating new parameter file $subdir/$newfl: $!\n";

  while(my $line = <ORIG>) {
    chomp $line;
    if($line !~ /^\s*\#/ || $line !~ /DoubleWarn/) {
      print NEW "$line\n";
    }
    elsif($line =~ /^(.*:)(\S+)\s+DoubleWarn=([^!]+)(!ALL)?\s*$/) {
      my $pre = $1;
      my $modelparam = $2;
      my $paramvalue = $3;

      print NEW "${pre}${modelparam} ${class}=";
      if($modelparam eq "initialWealthStr") {
	print NEW "$w0";
      }
      elsif($modelparam eq "generosityStr") {
	print NEW "$G";
      }
      elsif($modelparam eq "jackpotStr") {
	print NEW "$J";
      }
      else {
	print NEW "$paramvalue";
      }
      if($cparam ne "0") {
	print NEW "$cparam";
	$cparam = "0";
      }
      print NEW "\n";
    }
    else {
      die "Don't know what to do with this line in base parameter file $orig".
	":\n$line\n";
    }
  }
  print "Created parameter file $subdir/$newfl\n";
  close(NEW);
  close(ORIG);
}

