/*
    CharityWorld: I2.c
    Copyright (C) 2004  Macaulay Institute

    This file is part of CharityWorld, a simple agent-based model to
    demonstrate the effects of errors in floating point arithmetic on
    emergent outcomes.

    CharityWorld is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CharityWorld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/

/*

I2.c

For all integer jackpots J from start to finish, compute the
supply-awareness-conditions of edge, corner and centre agents not
neighbouring the winner immediately after the jackpot assuming a grid
of specified size. Check for floating point errors in the computation
of each.

Usage:

./I2 start finish size

*/

#include <stdio.h>		/* fprintf() */
#include <stdlib.h>		/* abort(), exit() */
#include <ieeefp.h>		/* fpget/setsticky() */
#include <math.h>		/* rint() */

typedef int BOOL;

typedef enum { CORNER = 0, EDGE, MIDDLE, N_AGENT_TYPES } agent_t;

const double n_nbrs[N_AGENT_TYPES] = { 3.0, 5.0, 8.0 };

#define NO 0
#define YES 1

void usage(char *cmd);
double neighbour_wealth_inc_self(double wealth, double n_nbrs, BOOL *err);

int main(int argc, char **argv) {
  double start, finish, size;
  double jackpot, wealth;
  

  if(argc != 4) {
    usage(argv[0]);
  }

  start = atof(argv[1]);
  finish = atof(argv[2]);
  size = atof(argv[3]);

  if(rint(start) != start || start <= 0.0) {
    fprintf(stderr, "Error: start argument (%g) must be a positive integer\n",
	    start);
    return 1;
  }
  if(rint(finish) != finish || finish <= start) {
    fprintf(stderr, "Error: finish argument (%g) must be an integer more than"
	    " start argument (%g)\n",
	    finish, start);
    return 1;
  }
  if(rint(size) != size || size < 9.0) {
    fprintf(stderr, "Error: grid size argument (%g) must be an integer >= 9\n",
	    size);
    return 1;
  }

  printf("Grid size:\t%g\n", size);

  printf("Jackpot\tAgent\tError\n");
  printf("\tC E M\tCw Ew Mw Ct Et Mt  T\n");

  fpsetsticky(0);
  for(jackpot = start; jackpot <= finish; jackpot += 1.0) {
    double ticket_price, nbr_wealth, awareness_threshold;
    BOOL wealth_err[N_AGENT_TYPES], threshold_err[N_AGENT_TYPES], ticket_err;
    BOOL awareness[N_AGENT_TYPES];
    agent_t i;

    if(fpgetsticky() != 0) {
      fprintf(stderr, "Floating point error incrementing jackpot!\n");
      abort();
    }

    fpsetsticky(0);

    ticket_price = jackpot;
    ticket_price /= size;

    ticket_err = (fpgetsticky() == 0 ? NO : YES);

    wealth = 0.0;
    wealth -= ticket_price;	/* Should always be exact */

    for(i = CORNER; i < N_AGENT_TYPES; i++) {
      nbr_wealth = neighbour_wealth_inc_self(wealth, n_nbrs[i],
					     &wealth_err[i]);
      fpsetsticky(0);

      awareness_threshold = (n_nbrs[i] + 1.0) * wealth;

      threshold_err[i] = (fpgetsticky() == 0 ? NO : YES);

      awareness[i] = (awareness_threshold > nbr_wealth ? YES : NO);
    }

    printf("%g\t%s %s %s\t%s  %s  %s  %s  %s  %s   %s\n",
	   jackpot,
	   awareness[CORNER] ? "T" : "F",
	   awareness[EDGE] ? "T" : "F",
	   awareness[MIDDLE] ? "T" : "F",
	   wealth_err[CORNER] ? "y" : "n",
	   wealth_err[EDGE] ? "y" : "n",
	   wealth_err[MIDDLE] ? "y" : "n",
	   threshold_err[CORNER] ? "y" : "n",
	   threshold_err[EDGE] ? "y" : "n",
	   threshold_err[MIDDLE] ? "y" : "n",
	   ticket_err ? "y" : "n");

    fpsetsticky(0);
  }

  return 0;
}

void usage(char *cmd) {
  printf("Usage: %s <start> <finish> <size>\n", cmd);
  printf("\nThis program runs through all integer jackpots from start to\n"
	 "finish, and checks the awareness condition of corner, edge and\n"
	 "middle agents immediately after the jackpot has been awarded,\n"
	 "assuming a grid of the specified size.\n");
  exit(1);
}

double neighbour_wealth_inc_self(double wealth, double n_nbrs, BOOL *err) {
  double i;
  double sum = wealth;

  fpsetsticky(0);

  for(i = 1.0; i <= n_nbrs; i += 1.0) {
    sum += wealth;
  }

  (*err) = (fpgetsticky() == 0 ? NO : YES);

  return sum;
}
